@@ -14,13 +14,13 @@
 ## |  Contact us for information about our services: info@rpact.com
 ## |  
 ## |  File name: test-f_design_fisher_combination_test.R
-## |  Creation date: 23 February 2022, 14:05:54
-## |  File version: $Revision: 5881 $
-## |  Last changed: $Date: 2022-02-24 12:35:06 +0100 (Do, 24 Feb 2022) $
+## |  Creation date: 12 August 2022, 09:09:51
+## |  File version: $Revision: 6485 $
+## |  Last changed: $Date: 2022-08-12 13:20:22 +0200 (Fr, 12 Aug 2022) $
 ## |  Last changed by: $Author: pahlke $
 ## |  
 
-context("Testing the Fisher Design Functionality")
+test_plan_section("Testing the Fisher Design Functionality")
 
 
 test_that("'getDesignFisher' with default parameters: parameters and results are as expected", {
